# PyTorch 0.3 中文文档 & 教程

![](../img/logo.svg)

> 维护组织：[@ApacheCN](https://github.com/apachecn)
> 
> 协议：[CC BY-NC-SA 4.0](http://creativecommons.org/licenses/by-nc-sa/4.0/)
> 
> 欢迎任何人参与和完善：一个人可以走的很快，但是一群人却可以走的更远。

+ [在线阅读](http://pytorch.apachecn.org)
+ [ApacheCN 学习资源](http://www.apachecn.org/)
+ [PyTorch 中文翻译组 | ApacheCN 713436582](http://shang.qq.com/wpa/qunwpa?idkey=349eb1bbaeeff1cf20408899cbe75669132ef145ff5ee6599f78a77dd144c367)

<!-- break -->

+ [在线阅读](https://pytorch.apachecn.org/docs/0.3/)
+ [PDF格式](https://www.gitbook.com/download/pdf/book/wizardforcel/pytorch-03-doc)
+ [EPUB格式](https://www.gitbook.com/download/epub/book/wizardforcel/pytorch-03-doc)
+ [MOBI格式](https://www.gitbook.com/download/mobi/book/wizardforcel/pytorch-03-doc)
+ [代码仓库](https://github.com/apachecn/pytorch-doc-zh)

## 目录结构

* [PyTorch 0.3 中文文档 & 教程](README.md)
* [中文教程](tut.md)
    * [初学者教程](beginner_tutorials.md)
        * [PyTorch 深度学习: 60 分钟极速入门教程](deep_learning_60min_blitz.md)
            * [PyTorch 是什么？](blitz_tensor_tutorial.md)
            * [自动求导: 自动微分](blitz_autograd_tutorial.md)
            * [神经网络](blitz_neural_networks_tutorial.md)
            * [训练一个分类器](blitz_cifar10_tutorial.md)
            * [可选: 数据并行](blitz_data_parallel_tutorial.md)
        * [PyTorch for former Torch users](former_torchies_tutorial.md)
            * [Tensors](former_torchies_tensor_tutorial.md)
            * [Autograd (自动求导)](former_torchies_autograd_tutorial.md)
            * [nn package](former_torchies_nn_tutorial.md)
            * [Multi-GPU examples](former_torchies_parallelism_tutorial.md)
        * [跟着例子学习 PyTorch](pytorch_with_examples.md)
            * [Warm-up: numpy](pytorch_with_examples_warm-up-numpy.md)
            * [PyTorch: Tensors](pytorch_with_examples_pytorch-tensors.md)
            * [PyTorch: 变量和autograd](pytorch_with_examples_pytorch-variables-and-autograd.md)
            * [PyTorch: 定义新的autograd函数](pytorch_with_examples_pytorch-defining-new-autograd-functions.md)
            * [TensorFlow: 静态图](pytorch_with_examples_tensorflow-static-graphs.md)
            * [PyTorch: nn包](pytorch_with_examples_pytorch-nn.md)
            * [PyTorch: optim包](pytorch_with_examples_pytorch-optim.md)
            * [PyTorch: 定制化nn模块](pytorch_with_examples_pytorch-custom-nn-modules.md)
            * [PyTorch: 动态控制流程 + 权重共享](pytorch_with_examples_pytorch-control-flow-weight-sharing.md)
        * [迁移学习教程](transfer_learning_tutorial.md)
        * [数据加载和处理教程](data_loading_tutorial.md)
        * [针对NLP的Pytorch深度学习](deep_learning_nlp_tutorial.md)
            * [PyTorch介绍](nlp_pytorch_tutorial.md)
            * [PyTorch深度学习](nlp_deep_learning_tutorial.md)
            * [词汇嵌入:编码词汇语义](nlp_word_embeddings_tutorial.md)
            * [序列模型和 LSTM 网络(长短记忆网络）](nlp_sequence_models_tutorial.md)
            * [高级教程: 作出动态决策和 Bi-LSTM CRF](nlp_advanced_tutorial.md)
    * [中级教程](intermediate_tutorials.md)
        * [用字符级RNN分类名称](char_rnn_classification_tutorial.md)
        * [基与字符级RNN(Char-RNN）的人名生成](char_rnn_generation_tutorial.md)
        * [用基于注意力机制的seq2seq神经网络进行翻译](seq2seq_translation_tutorial.md)
        * [强化学习(DQN）教程](reinforcement_q_learning.md)
        * [Writing Distributed Applications with PyTorch](dist_tuto.md)
        * [空间转换网络 (Spatial Transformer Networks) 教程](spatial_transformer_tutorial.md)
    * [高级教程](advanced_tutorials.md)
        * [用 PyTorch 做 神经转换 (Neural Transfer)](neural_style_tutorial.md)
        * [使用 numpy 和 scipy 创建扩展](numpy_extensions_tutorial.md)
        * [使用 ONNX 将模型从 PyTorch 迁移到 Caffe2 和 Mobile](super_resolution_with_caffe2.md)
        * [为 pytorch 自定义 C 扩展](c_extension.md)
* [中文文档](doc.md)
    * [介绍](notes.md)
        * [自动求导机制](notes_autograd.md)
        * [广播语义](notes_broadcasting.md)
        * [CUDA 语义](notes_cuda.md)
        * [扩展 PyTorch](notes_extending.md)
        * [多进程的最佳实践](notes_multiprocessing.md)
        * [序列化语义](notes_serialization.md)
    * [Package 参考](package_reference.md)
        * [torch](torch.md)
        * [torch.Tensor](tensors.md)
        * [torch.sparse](sparse.md)
        * [torch.Storage](storage.md)
        * [torch.nn](nn.md)
        * [torch.optim](optim.md)
        * [Automatic differentiation package - torch.autograd](autograd.md)
        * [Probability distributions - torch.distributions](distributions.md)
        * [Multiprocessing package - torch.multiprocessing](multiprocessing.md)
        * [Distributed communication package - torch.distributed](distributed.md)
        * [Legacy package - torch.legacy](legacy.md)
        * [torch.cuda](cuda.md)
        * [torch.utils.ffi](ffi.md)
        * [torch.utils.data](data.md)
        * [torch.utils.model_zoo](model_zoo.md)
        * [torch.onnx](onnx.md)
    * [torchvision 参考](torchvision_reference.md)
        * [torchvision](torchvision.md)
        * [torchvision.datasets](datasets.md)
        * [torchvision.models](models.md)
        * [torchvision.transforms](transforms.md)
        * [torchvision.utils](utils.md)

